@@ -2,6 +2,7 @@
  *  Extract RAR archives
  *
  *  Copyright (C) 2005-2006 trog@uncon.org
+ *  Patches added by Sourcefire, Inc. Copyright (C) 2007-2013
  *
  *  This code is based on the work of Alexander L. Roshal (C)
  *
@@ -15,6 +16,10 @@
  *
  */
 
+#if HAVE_CONFIG_H
+#include "clamav-config.h"
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -153,9 +158,18 @@ static void unp_write_data(unpack_data_t *unpack_data, uint8_t *data, int size)
 {
 	int ret;
 	rar_dbgmsg("in unp_write_data length=%d\n", size);
+
+	unpack_data->true_size += size;
+	unpack_data->unp_crc = rar_crc(unpack_data->unp_crc, data, size);
+	if(unpack_data->max_size) {
+	    if(unpack_data->written_size >= unpack_data->max_size)
+		return;
+
+	    if(unpack_data->written_size + size > unpack_data->max_size)
+		size = unpack_data->max_size - unpack_data->written_size;
+	}
 	if((ret = write(unpack_data->ofd, data, size)) > 0)
 	    unpack_data->written_size += ret;
-	unpack_data->unp_crc = rar_crc(unpack_data->unp_crc, data, size);
 }
 
 static void unp_write_area(unpack_data_t *unpack_data, unsigned int start_ptr, unsigned int end_ptr)
@@ -434,7 +448,7 @@ static int read_tables(int fd, unpack_data_t *unpack_data)
 		}
 	}
 	rar_make_decode_tables(bit_length,(struct Decode *)&unpack_data->BD,BC);
-	
+	memset(table, 0, sizeof(table));
 	for (i=0;i<table_size;) {
 		if (unpack_data->in_addr > unpack_data->read_top-5) {
 			if (!rar_unp_read_buf(fd, unpack_data)) {
@@ -455,7 +469,8 @@ static int read_tables(int fd, unpack_data_t *unpack_data)
 				rar_addbits(unpack_data, 7);
 			}
 			while (n-- > 0 && i < table_size) {
-				table[i] = table[i-1];
+				if (i>0)
+					table[i] = table[i-1];
 				i++;
 			}
 		} else {
@@ -637,7 +652,7 @@ static int add_vm_code(unpack_data_t *unpack_data, unsigned int first_byte,
 	}
 	if (new_filter) {
 		vm_codesize = rarvm_read_data(&rarvm_input);
-		if (vm_codesize >= 0x1000 || vm_codesize == 0 || (vm_codesize > rarvm_input.buf_size)) {
+		if (vm_codesize >= 0x1000 || vm_codesize == 0 || (vm_codesize > rarvm_input.buf_size) || vm_codesize < 0) {
 			rar_dbgmsg("ERROR: vm_codesize=0x%x buf_size=0x%x\n", vm_codesize, rarvm_input.buf_size);
 			return FALSE;
 		}
@@ -746,6 +761,7 @@ static int read_vm_code(unpack_data_t *unpack_data, int fd)
 	for (i=0 ; i < length ; i++) {
 		if (unpack_data->in_addr >= unpack_data->read_top-1 &&
 				!rar_unp_read_buf(fd, unpack_data) && i<length-1) {
+			free(vmcode);
 			return FALSE;
 		}
 		vmcode[i] = rar_getbits(unpack_data) >> 8;
@@ -827,6 +843,7 @@ void rar_unpack_init_data(int solid, unpack_data_t *unpack_data)
 	unpack_data->read_top = 0;
 	unpack_data->read_border = 0;
 	unpack_data->written_size = 0;
+	unpack_data->true_size = 0;
 	rarvm_init(&unpack_data->rarvm_data);
 	unpack_data->unp_crc = 0xffffffff;
 	
@@ -917,7 +934,7 @@ static int rar_unpack29(int fd, int solid, unpack_data_t *unpack_data)
 					continue;
 				}
 				if (next_ch == 4) {
-					unsigned int length;
+					unsigned int length = 0;
 					distance = 0;
 					failed = FALSE;
 					for (i=0 ; i < 4 && !failed; i++) {
@@ -1067,7 +1084,6 @@ static int rar_unpack29(int fd, int solid, unpack_data_t *unpack_data)
 	if (retval) {
 		unp_write_buf(unpack_data);
 	}
-	rar_dbgmsg("Finished length: %ld\n", unpack_data->written_size);
 	return retval;
 }
 
@@ -1088,11 +1104,17 @@ int rar_unpack(int fd, int method, int solid, unpack_data_t *unpack_data)
 	default:
 		retval = rar_unpack29(fd, solid, unpack_data);
 		if(retval == FALSE) {
+		    rarvm_free(&unpack_data->rarvm_data);
 		    retval = rar_unpack20(fd, solid, unpack_data);
-		    if(retval == FALSE)
+		    if(retval == FALSE) {
+			rarvm_free(&unpack_data->rarvm_data);
 			retval = rar_unpack15(fd, solid, unpack_data);
+		    }
 		}
 		break;
 	}
+	rar_dbgmsg("Written size: %ld\n", unpack_data->written_size);
+	rar_dbgmsg("True size: %ld\n", unpack_data->true_size);
+
 	return retval;
 }
